@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,12 +18,12 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sslgen.c,v 1.37 2008-05-09 11:27:55 mmarek Exp $
+ * $Id: sslgen.c,v 1.48 2009-05-04 21:57:14 bagder Exp $
  ***************************************************************************/
 
-/* This file is for "generic" SSL functions that all libcurl internals should
-   use. It is responsible for calling the proper 'ossl' function in ssluse.c
-   (OpenSSL based) or the 'gtls' function in gtls.c (GnuTLS based).
+/* This file is for implementing all "generic" SSL functions that all libcurl
+   internals should use. It is then responsible for calling the proper
+   "backend" function.
 
    SSL-functions in libcurl should call functions in this source file, and not
    to any specific SSL-layer.
@@ -33,6 +33,9 @@
    Curl_gtls_ - prefix for GnuTLS ones
    Curl_nss_ - prefix for NSS ones
 
+   Note that this source code uses curlssl_* functions, and they are all
+   defines/macros #defined by the lib-specific header files.
+
    "SSL/TLS Strong Encryption: An Introduction"
    http://httpd.apache.org/docs-2.0/ssl/ssl_intro.html
 */
@@ -54,22 +57,18 @@
 #include "nssg.h"   /* NSS versions */
 #include "qssl.h"   /* QSOSSL versions */
 #include "sendf.h"
-#include "strequal.h"
+#include "rawstr.h"
 #include "url.h"
-#include "memory.h"
+#include "curl_memory.h"
+#include "progress.h"
 /* The last #include file should be: */
 #include "memdebug.h"
 
-/* "global" init done? */
-static bool init_ssl=FALSE;
-
-static bool safe_strequal(char* str1, char* str2);
-
 static bool safe_strequal(char* str1, char* str2)
 {
   if(str1 && str2)
     /* both pointers point to something then compare them */
-    return (bool)(0 != strequal(str1, str2));
+    return (bool)(0 != Curl_raw_equal(str1, str2));
   else
     /* if both pointers are NULL then treat them as equal */
     return (bool)(!str1 && !str2);
@@ -143,6 +142,11 @@ void Curl_free_ssl_config(struct ssl_config_data* sslc)
   Curl_safefree(sslc->random_file);
 }
 
+#ifdef USE_SSL
+
+/* "global" init done? */
+static bool init_ssl=FALSE;
+
 /**
  * Global SSL init
  *
@@ -156,24 +160,7 @@ int Curl_ssl_init(void)
     return 1;
   init_ssl = TRUE; /* never again */
 
-#ifdef USE_SSLEAY
-  return Curl_ossl_init();
-#else
-#ifdef USE_GNUTLS
-  return Curl_gtls_init();
-#else
-#ifdef USE_NSS
-  return Curl_nss_init();
-#else
-#ifdef USE_QSOSSL
-  return Curl_qsossl_init();
-#else
-  /* no SSL support */
-  return 1;
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  return curlssl_init();
 }
 
 
@@ -182,21 +169,7 @@ void Curl_ssl_cleanup(void)
 {
   if(init_ssl) {
     /* only cleanup if we did a previous init */
-#ifdef USE_SSLEAY
-    Curl_ossl_cleanup();
-#else
-#ifdef USE_GNUTLS
-    Curl_gtls_cleanup();
-#else
-#ifdef USE_NSS
-    Curl_nss_cleanup();
-#else
-#ifdef USE_QSOSSL
-    Curl_qsossl_cleanup();
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+    curlssl_cleanup();
     init_ssl = FALSE;
   }
 }
@@ -204,67 +177,38 @@ void Curl_ssl_cleanup(void)
 CURLcode
 Curl_ssl_connect(struct connectdata *conn, int sockindex)
 {
-#ifdef USE_SSL
-  /* mark this is being ssl enabled from here on. */
+  CURLcode res;
+  /* mark this is being ssl-enabled from here on. */
   conn->ssl[sockindex].use = TRUE;
   conn->ssl[sockindex].state = ssl_connection_negotiating;
 
-#ifdef USE_SSLEAY
-  return Curl_ossl_connect(conn, sockindex);
-#else
-#ifdef USE_GNUTLS
-  return Curl_gtls_connect(conn, sockindex);
-#else
-#ifdef USE_NSS
-  return Curl_nss_connect(conn, sockindex);
-#else
-#ifdef USE_QSOSSL
-  return Curl_qsossl_connect(conn, sockindex);
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  res = curlssl_connect(conn, sockindex);
 
-#else
-  /* without SSL */
-  (void)conn;
-  (void)sockindex;
-  return CURLE_OK;
-#endif /* USE_SSL */
+  if(!res)
+    Curl_pgrsTime(conn->data, TIMER_APPCONNECT); /* SSL is connected */
+
+  return res;
 }
 
 CURLcode
 Curl_ssl_connect_nonblocking(struct connectdata *conn, int sockindex,
                              bool *done)
 {
-#if defined(USE_SSL) && defined(USE_SSLEAY)
-  /* mark this is being ssl enabled from here on. */
+#ifdef curlssl_connect_nonblocking
+  CURLcode res;
+  /* mark this is being ssl requested from here on. */
   conn->ssl[sockindex].use = TRUE;
-  return Curl_ossl_connect_nonblocking(conn, sockindex, done);
-
+  res = curlssl_connect_nonblocking(conn, sockindex, done);
+  if(!res && *done == TRUE)
+    Curl_pgrsTime(conn->data, TIMER_APPCONNECT); /* SSL is connected */
+  return res;
 #else
-#ifdef USE_NSS
   *done = TRUE; /* fallback to BLOCKING */
   conn->ssl[sockindex].use = TRUE;
-  return Curl_nss_connect(conn, sockindex);
-#else
-#ifdef USE_QSOSSL
-  *done = TRUE; /* fallback to BLOCKING */
-  conn->ssl[sockindex].use = TRUE;
-  return Curl_qsossl_connect(conn, sockindex);
-#else
-  /* not implemented!
-     fallback to BLOCKING call. */
-  *done = TRUE;
-  conn->ssl[sockindex].use = TRUE;
-  return Curl_ssl_connect(conn, sockindex);
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_SSLEAY */
+  return curlssl_connect(conn, sockindex);
+#endif /* non-blocking connect support */
 }
 
-#ifdef USE_SSL
-
 /*
  * Check if there's a session ID for the given connection in the cache, and if
  * there's one suitable, it is provided. Returns TRUE when no entry matched.
@@ -286,7 +230,7 @@ int Curl_ssl_getsessionid(struct connectdata *conn,
     if(!check->sessionid)
       /* not session ID means blank entry */
       continue;
-    if(curl_strequal(conn->host.name, check->name) &&
+    if(Curl_raw_equal(conn->host.name, check->name) &&
        (conn->remote_port == check->remote_port) &&
        Curl_ssl_config_matches(&conn->ssl_config, &check->ssl_config)) {
       /* yes, we have a session ID! */
@@ -311,21 +255,8 @@ static int kill_session(struct curl_ssl_session *session)
     /* defensive check */
 
     /* free the ID the SSL-layer specific way */
-#ifdef USE_SSLEAY
-    Curl_ossl_session_free(session->sessionid);
-#else
-#ifdef USE_GNUTLS
-    Curl_gtls_session_free(session->sessionid);
-#else
-#ifdef USE_QSOSSL
-    /* No session handling for QsoSSL. */
-#else
-#ifdef USE_NSS
-    /* NSS has its own session ID cache */
-#endif /* USE_NSS */
-#endif /* USE_QSOSSL */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+    curlssl_session_free(session->sessionid);
+
     session->sessionid=NULL;
     session->age = 0; /* fresh */
 
@@ -340,6 +271,22 @@ static int kill_session(struct curl_ssl_session *session)
     return 1;
 }
 
+/*
+ * Delete the given session ID from the cache.
+ */
+void Curl_ssl_delsessionid(struct connectdata *conn, void *ssl_sessionid)
+{
+  int i;
+  for(i=0; i< conn->data->set.ssl.numsessions; i++) {
+    struct curl_ssl_session *check = &conn->data->state.session[i];
+
+    if (check->sessionid == ssl_sessionid) {
+      kill_session(check);
+      break;
+    }
+  }
+}
+
 /*
  * Store session id in the session cache. The ID passed on to this function
  * must already have been extracted and allocated the proper way for the SSL
@@ -398,11 +345,8 @@ CURLcode Curl_ssl_addsessionid(struct connectdata *conn,
 }
 
 
-#endif /* USE_SSL */
-
 void Curl_ssl_close_all(struct SessionHandle *data)
 {
-#ifdef USE_SSL
   long i;
   /* kill the session ID cache */
   if(data->state.session) {
@@ -414,64 +358,20 @@ void Curl_ssl_close_all(struct SessionHandle *data)
     free(data->state.session);
     data->state.session = NULL;
   }
-#ifdef USE_SSLEAY
-  Curl_ossl_close_all(data);
-#else
-#ifdef USE_GNUTLS
-  Curl_gtls_close_all(data);
-#else
-#ifdef USE_NSS
-  Curl_nss_close_all(data);
-#else
-#ifdef USE_QSOSSL
-  Curl_qsossl_close_all(data);
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
-#else /* USE_SSL */
-  (void)data;
-#endif /* USE_SSL */
+
+  curlssl_close_all(data);
 }
 
 void Curl_ssl_close(struct connectdata *conn, int sockindex)
 {
   DEBUGASSERT((sockindex <= 1) && (sockindex >= -1));
-
-#ifdef USE_SSLEAY
-  Curl_ossl_close(conn, sockindex);
-#endif /* USE_SSLEAY */
-#ifdef USE_GNUTLS
-  Curl_gtls_close(conn, sockindex);
-#endif /* USE_GNUTLS */
-#ifdef USE_NSS
-  Curl_nss_close(conn, sockindex);
-#endif /* USE_NSS */
-#ifdef USE_QSOSSL
-  Curl_qsossl_close(conn, sockindex);
-#endif /* USE_QSOSSL */
-#ifndef USE_SSL
-  (void)conn;
-  (void)sockindex;
-#endif /* !USE_SSL */
+  curlssl_close(conn, sockindex);
 }
 
 CURLcode Curl_ssl_shutdown(struct connectdata *conn, int sockindex)
 {
-#ifdef USE_SSLEAY
-  if(Curl_ossl_shutdown(conn, sockindex))
+  if(curlssl_shutdown(conn, sockindex))
     return CURLE_SSL_SHUTDOWN_FAILED;
-#else
-#ifdef USE_GNUTLS
-  if(Curl_gtls_shutdown(conn, sockindex))
-    return CURLE_SSL_SHUTDOWN_FAILED;
-#else
-#ifdef USE_QSOSSL
-  if(Curl_qsossl_shutdown(conn, sockindex))
-    return CURLE_SSL_SHUTDOWN_FAILED;
-#endif /* USE_QSOSSL */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
 
   conn->ssl[sockindex].use = FALSE; /* get back to ordinary socket usage */
   conn->ssl[sockindex].state = ssl_connection_none;
@@ -479,100 +379,24 @@ CURLcode Curl_ssl_shutdown(struct connectdata *conn, int sockindex)
   return CURLE_OK;
 }
 
-/* Selects an (Open)SSL crypto engine
+/* Selects an SSL crypto engine
  */
 CURLcode Curl_ssl_set_engine(struct SessionHandle *data, const char *engine)
 {
-#ifdef USE_SSLEAY
-  return Curl_ossl_set_engine(data, engine);
-#else
-#ifdef USE_GNUTLS
-  /* FIX: add code here */
-  (void)data;
-  (void)engine;
-  return CURLE_FAILED_INIT;
-#else
-#ifdef USE_NSS
-  /* NSS doesn't set an engine this way */
-  (void)data;
-  (void)engine;
-  return CURLE_FAILED_INIT;
-#else
-#ifdef USE_QSOSSL
-  /* QSOSSL doesn't set an engine this way */
-  (void)data;
-  (void)engine;
-  return CURLE_FAILED_INIT;
-#else
-  /* no SSL layer */
-  (void)data;
-  (void)engine;
-  return CURLE_FAILED_INIT;
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  return curlssl_set_engine(data, engine);
 }
 
-/* Selects an (Open?)SSL crypto engine
+/* Selects the default SSL crypto engine
  */
 CURLcode Curl_ssl_set_engine_default(struct SessionHandle *data)
 {
-#ifdef USE_SSLEAY
-  return Curl_ossl_set_engine_default(data);
-#else
-#ifdef USE_GNUTLS
-  /* FIX: add code here */
-  (void)data;
-  return CURLE_FAILED_INIT;
-#else
-#ifdef USE_NSS
-  /* A no-op for NSS */
-  (void)data;
-  return CURLE_FAILED_INIT;
-#else
-#ifdef USE_QSOSSL
-  /* A no-op for QSOSSL */
-  (void)data;
-  return CURLE_FAILED_INIT;
-#else
-  /* No SSL layer */
-  (void)data;
-  return CURLE_FAILED_INIT;
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  return curlssl_set_engine_default(data);
 }
 
 /* Return list of OpenSSL crypto engine names. */
 struct curl_slist *Curl_ssl_engines_list(struct SessionHandle *data)
 {
-#ifdef USE_SSLEAY
-  return Curl_ossl_engines_list(data);
-#else
-#ifdef USE_GNUTLS
-  /* FIX: add code here? */
-  (void)data;
-  return NULL;
-#else
-#ifdef USE_NSS
-  /* In theory we could return the PKCS#11 modules loaded but that
-   * would just confuse things */
-  (void)data;
-  return NULL;
-#else
-#ifdef USE_QSOSSL
-  /* No engine support in QSOSSL. */
-  (void)data;
-  return NULL;
-#else
-  (void)data;
-  return NULL;
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  return curlssl_engines_list(data);
 }
 
 /* return number of sent (non-SSL) bytes */
@@ -581,27 +405,7 @@ ssize_t Curl_ssl_send(struct connectdata *conn,
                       const void *mem,
                       size_t len)
 {
-#ifdef USE_SSLEAY
-  return Curl_ossl_send(conn, sockindex, mem, len);
-#else
-#ifdef USE_GNUTLS
-  return Curl_gtls_send(conn, sockindex, mem, len);
-#else
-#ifdef USE_NSS
-  return Curl_nss_send(conn, sockindex, mem, len);
-#else
-#ifdef USE_QSOSSL
-  return Curl_qsossl_send(conn, sockindex, mem, len);
-#else
-  (void)conn;
-  (void)sockindex;
-  (void)mem;
-  (void)len;
-  return 0;
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  return curlssl_send(conn, sockindex, mem, len);
 }
 
 /* return number of received (decrypted) bytes */
@@ -615,25 +419,10 @@ ssize_t Curl_ssl_recv(struct connectdata *conn, /* connection data */
                       char *mem,                /* store read data here */
                       size_t len)               /* max amount to read */
 {
-#ifdef USE_SSL
   ssize_t nread;
   bool block = FALSE;
 
-#ifdef USE_SSLEAY
-  nread = Curl_ossl_recv(conn, sockindex, mem, len, &block);
-#else
-#ifdef USE_GNUTLS
-  nread = Curl_gtls_recv(conn, sockindex, mem, len, &block);
-#else
-#ifdef USE_NSS
-  nread = Curl_nss_recv(conn, sockindex, mem, len, &block);
-#else
-#ifdef USE_QSOSSL
-  nread = Curl_qsossl_recv(conn, sockindex, mem, len, &block);
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  nread = curlssl_recv(conn, sockindex, mem, len, &block);
   if(nread == -1) {
     if(!block)
       return 0; /* this is a true error, not EWOULDBLOCK */
@@ -642,14 +431,6 @@ ssize_t Curl_ssl_recv(struct connectdata *conn, /* connection data */
   }
 
   return nread;
-
-#else /* USE_SSL */
-  (void)conn;
-  (void)sockindex;
-  (void)mem;
-  (void)len;
-  return 0;
-#endif /* USE_SSL */
 }
 
 
@@ -659,15 +440,13 @@ ssize_t Curl_ssl_recv(struct connectdata *conn, /* connection data */
  */
 CURLcode Curl_ssl_initsessions(struct SessionHandle *data, long amount)
 {
-#ifdef USE_SSL
   struct curl_ssl_session *session;
 
   if(data->state.session)
     /* this is just a precaution to prevent multiple inits */
     return CURLE_OK;
 
-  session = (struct curl_ssl_session *)
-    calloc(sizeof(struct curl_ssl_session), amount);
+  session = calloc(sizeof(struct curl_ssl_session), amount);
   if(!session)
     return CURLE_OUT_OF_MEMORY;
 
@@ -675,39 +454,14 @@ CURLcode Curl_ssl_initsessions(struct SessionHandle *data, long amount)
   data->set.ssl.numsessions = amount;
   data->state.session = session;
   data->state.sessionage = 1; /* this is brand new */
-#else
-  /* without SSL, do nothing */
-  (void)data;
-  (void)amount;
-#endif
-
   return CURLE_OK;
 }
 
 size_t Curl_ssl_version(char *buffer, size_t size)
 {
-#ifdef USE_SSLEAY
-  return Curl_ossl_version(buffer, size);
-#else
-#ifdef USE_GNUTLS
-  return Curl_gtls_version(buffer, size);
-#else
-#ifdef USE_NSS
-  return Curl_nss_version(buffer, size);
-#else
-#ifdef USE_QSOSSL
-  return Curl_qsossl_version(buffer, size);
-#else
-  (void)buffer;
-  (void)size;
-  return 0; /* no SSL support */
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_GNUTLS */
-#endif /* USE_SSLEAY */
+  return curlssl_version(buffer, size);
 }
 
-
 /*
  * This function tries to determine connection status.
  *
@@ -718,35 +472,25 @@ size_t Curl_ssl_version(char *buffer, size_t size)
  */
 int Curl_ssl_check_cxn(struct connectdata *conn)
 {
-#ifdef USE_SSLEAY
-  return Curl_ossl_check_cxn(conn);
-#else
-#ifdef USE_NSS
-  return Curl_nss_check_cxn(conn);
-#else
-#ifdef USE_QSOSSL
-  return Curl_qsossl_check_cxn(conn);
-#else
-  (void)conn;
-  /* TODO: we lack implementation of this for GnuTLS */
-  return -1; /* connection status unknown */
-#endif /* USE_QSOSSL */
-#endif /* USE_NSS */
-#endif /* USE_SSLEAY */
+  return curlssl_check_cxn(conn);
 }
 
 bool Curl_ssl_data_pending(const struct connectdata *conn,
                            int connindex)
 {
-#ifdef USE_SSLEAY
-  /* OpenSSL-specific */
-  if(conn->ssl[connindex].handle)
-    /* SSL is in use */
-    return (bool)(0 != SSL_pending(conn->ssl[connindex].handle));
-#else
-  (void)conn;
-  (void)connindex;
-#endif
-  return FALSE; /* nothing pending */
-
+  return curlssl_data_pending(conn, connindex);
 }
+
+void Curl_ssl_free_certinfo(struct SessionHandle *data)
+{
+  int i;
+  struct curl_certinfo *ci = &data->info.certs;
+  if(ci->num_of_certs) {
+    /* free all individual lists used */
+    for(i=0; i<ci->num_of_certs; i++)
+      curl_slist_free_all(ci->certinfo[i]);
+    free(ci->certinfo); /* free the actual array too */
+    ci->num_of_certs = 0;
+  }
+}
+#endif /* USE_SSL */
