@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2009, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: ccsidcurl.c,v 1.12 2008-10-17 13:17:41 patrickm Exp $
+ * $Id: ccsidcurl.c,v 1.16 2009-08-31 15:22:16 patrickm Exp $
  *
  ***************************************************************************/
 
@@ -32,8 +32,8 @@
 
 #pragma enum(int)
 
-#include <curl/curl.h>
-#include <curl/mprintf.h>
+#include "curl.h"
+#include "mprintf.h"
 #include "urldata.h"
 #include "url.h"
 #include "getinfo.h"
@@ -1030,7 +1030,7 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
   if (testwarn) {
     testwarn = 0;
 
-    if ((int) STRING_LAST != (int) STRING_PROXYPASSWORD + 1)
+    if ((int) STRING_LAST != (int) STRING_SOCKS5_GSSAPI_SERVICE + 1)
       curl_mfprintf(stderr,
        "*** WARNING: curl_easy_setopt_ccsid() should be reworked ***\n");
     }
@@ -1079,6 +1079,10 @@ curl_easy_setopt_ccsid(CURL * curl, CURLoption tag, ...)
   case CURLOPT_PASSWORD:
   case CURLOPT_PROXYUSERNAME:
   case CURLOPT_PROXYPASSWORD:
+  case CURLOPT_NOPROXY:
+  /* SSH2 not (yet) implemented on OS400. */
+  /* case CURLOPT_SSH_KNOWNHOSTS: */
+  case CURLOPT_SOCKS5_GSSAPI_SERVICE:
     s = va_arg(arg, char *);
     ccsid = va_arg(arg, unsigned int);
 
